// Copyright 2022, FOSS-VG Developers and Contributers
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU Affero General Public License as published
// by the Free Software Foundation, version 3.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// version 3 along with this program.
// If not, see https://www.gnu.org/licenses/agpl-3.0.en.html

#include <string>
#include <iostream>
#include <vector>
#include "assert.h++"
#include "../lib/error.h++"

#include "../lib/cli.h++"

int main() {
    std::cout << "################################################################################" << std::endl;
    std::cout << "CLI argument parser tests" << std::endl;
    std::cout << "################################################################################" << std::endl;

    // Valid parameter test ############################################
    // This test exercises the basic functionality of the arguments parser
    // with many variations of valid input on a single command line.
    //
    // simulated command line:
    // test -0 -ab -12345 --long-flag -cconcatenated -d separate-value -efdouble-concatenated "positional argument 0" -gh concatenated-separate-value --long-argument-with-value-included="included value" --long-argument-with-value-separated "separate value" "positional argument 1" "positional argument 2"

    std::vector<CLI::Flag> validTestFlags;
    validTestFlags.push_back(CLI::Flag('0', "00000", "a short flag on its own"));
    validTestFlags.push_back(CLI::Flag('a', "aaaaa", "concatenated short flags"));
    validTestFlags.push_back(CLI::Flag('b', "bbbbb", "concatenated short flags"));
    validTestFlags.push_back(CLI::Flag('1', "11111", "more than two concatenated short flags"));
    validTestFlags.push_back(CLI::Flag('2', "22222", "more than two concatenated short flags"));
    validTestFlags.push_back(CLI::Flag('3', "33333", "more than two concatenated short flags"));
    validTestFlags.push_back(CLI::Flag('4', "44444", "more than two concatenated short flags"));
    validTestFlags.push_back(CLI::Flag('5', "55555", "more than two concatenated short flags"));
    validTestFlags.push_back(CLI::Flag('l', "long-flag", "long flags"));
    validTestFlags.push_back(CLI::Flag('e', "eeeee", "short flags concatenated with an argument that is concatenated with its value"));
    validTestFlags.push_back(CLI::Flag('g', "ggggg", "short flags concatenated with an argument that has a separate value"));
    validTestFlags.push_back(CLI::Flag('6', "66666", "unused flag"));

    std::vector<CLI::UnpositionalArgument> validTestUnpositionalArguments;
    validTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('c', "ccccc", "VALUE", "short argument concatenated with its value"));
    validTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('d', "ddddd", " VALUE", "short argument with separate value"));
    validTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('f', "fffff", "VALUE", "short argument concatenated with a flag and its value"));
    validTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('h', "hhhhh", " VALUE", "short argument concatenated with a flag with separate value"));
    validTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('x', "long-argument-with-value-included", "VALUE", "long argument with its value included using ="));
    validTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('y', "long-argument-with-value-separated", " VALUE", "long argument with separate value"));
    validTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('z', "zzzzz", "NOPE", "unused argument"));

    std::vector<CLI::PositionalArgument> validTestPositionalArguments;
    validTestPositionalArguments.push_back(CLI::PositionalArgument("argument0", "positional argument between optional parameters"));
    validTestPositionalArguments.push_back(CLI::PositionalArgument("argument1", "positional argument"));
    validTestPositionalArguments.push_back(CLI::PositionalArgument("argument2", "positional argument"));

    const char** validTestParameterList = new const char*[17];
    validTestParameterList[ 0] = "test";
    validTestParameterList[ 1] = "-0";
    validTestParameterList[ 2] = "-ab";
    validTestParameterList[ 3] = "-12345";
    validTestParameterList[ 4] = "--long-flag";
    validTestParameterList[ 5] = "-cconcatenated";
    validTestParameterList[ 6] = "-d";
    validTestParameterList[ 7] = "separate-value";
    validTestParameterList[ 8] = "-efdouble-concatenated";
    validTestParameterList[ 9] = "positional argument 0";
    validTestParameterList[10] = "-gh";
    validTestParameterList[11] = "concatenated-separate-value";
    validTestParameterList[12] = "--long-argument-with-value-included=included value";
    validTestParameterList[13] = "--long-argument-with-value-separated";
    validTestParameterList[14] = "separate value";
    validTestParameterList[15] = "positional argument 1";
    validTestParameterList[16] = "positional argument 2";
    int validTestParameterCount = 17;

    CLI::ArgumentsParser validTestParameterParser = CLI::ArgumentsParser(validTestParameterCount, validTestParameterList, validTestFlags, validTestUnpositionalArguments, validTestPositionalArguments);

    ASSERT(!validTestParameterParser.wrongUsage);
    ASSERT(validTestParameterParser.programName == std::string("test"));
    ASSERT(!validTestParameterParser.getFlag('0').isError);
    ASSERT(validTestParameterParser.getFlag('0').value);
    ASSERT(validTestParameterParser.getFlag(std::string("00000")).value);
    ASSERT(!validTestParameterParser.getFlag('a').isError);
    ASSERT(validTestParameterParser.getFlag('a').value);
    ASSERT(validTestParameterParser.getFlag(std::string("aaaaa")).value);
    ASSERT(!validTestParameterParser.getFlag('b').isError);
    ASSERT(validTestParameterParser.getFlag('b').value);
    ASSERT(validTestParameterParser.getFlag(std::string("bbbbb")).value);
    ASSERT(!validTestParameterParser.getFlag('1').isError);
    ASSERT(validTestParameterParser.getFlag('1').value);
    ASSERT(validTestParameterParser.getFlag(std::string("11111")).value);
    ASSERT(!validTestParameterParser.getFlag('2').isError);
    ASSERT(validTestParameterParser.getFlag('2').value);
    ASSERT(validTestParameterParser.getFlag(std::string("22222")).value);
    ASSERT(!validTestParameterParser.getFlag('3').isError);
    ASSERT(validTestParameterParser.getFlag('3').value);
    ASSERT(validTestParameterParser.getFlag(std::string("33333")).value);
    ASSERT(!validTestParameterParser.getFlag('4').isError);
    ASSERT(validTestParameterParser.getFlag('4').value);
    ASSERT(validTestParameterParser.getFlag(std::string("44444")).value);
    ASSERT(!validTestParameterParser.getFlag('5').isError);
    ASSERT(validTestParameterParser.getFlag('5').value);
    ASSERT(validTestParameterParser.getFlag(std::string("55555")).value);
    ASSERT(!validTestParameterParser.getFlag(std::string("long-flag")).isError);
    ASSERT(validTestParameterParser.getFlag('l').value);
    ASSERT(validTestParameterParser.getFlag(std::string("long-flag")).value);
    ASSERT(!validTestParameterParser.getFlag('6').isError);
    ASSERT(!validTestParameterParser.getFlag('6').value);
    ASSERT(!validTestParameterParser.getFlag(std::string("66666")).value);
    ASSERT(!validTestParameterParser.getUnpositionalArgument('c').isError);
    ASSERT(validTestParameterParser.getUnpositionalArgument('c').value==std::string("concatenated"));
    ASSERT(validTestParameterParser.getUnpositionalArgument(std::string("ccccc")).value==std::string("concatenated"));
    ASSERT(!validTestParameterParser.getUnpositionalArgument('d').isError);
    ASSERT(validTestParameterParser.getUnpositionalArgument('d').value==std::string("separate-value"));
    ASSERT(validTestParameterParser.getUnpositionalArgument(std::string("ddddd")).value==std::string("separate-value"));
    ASSERT(!validTestParameterParser.getFlag('e').isError);
    ASSERT(validTestParameterParser.getFlag('e').value);
    ASSERT(validTestParameterParser.getFlag(std::string("eeeee")).value);
    ASSERT(!validTestParameterParser.getUnpositionalArgument('f').isError);
    ASSERT(validTestParameterParser.getUnpositionalArgument('f').value==std::string("double-concatenated"));
    ASSERT(validTestParameterParser.getUnpositionalArgument(std::string("fffff")).value==std::string("double-concatenated"));
    ASSERT(!validTestParameterParser.getPositionalArgument(0).isError);
    ASSERT(validTestParameterParser.getPositionalArgument(0).value=="positional argument 0");
    ASSERT(!validTestParameterParser.getFlag('g').isError);
    ASSERT(validTestParameterParser.getFlag('g').value);
    ASSERT(validTestParameterParser.getFlag(std::string("ggggg")).value);
    ASSERT(!validTestParameterParser.getUnpositionalArgument('h').isError);
    ASSERT(validTestParameterParser.getUnpositionalArgument('h').value==std::string("concatenated-separate-value"));
    ASSERT(validTestParameterParser.getUnpositionalArgument(std::string("hhhhh")).value==std::string("concatenated-separate-value"));
    ASSERT(!validTestParameterParser.getUnpositionalArgument('x').isError);
    ASSERT(validTestParameterParser.getUnpositionalArgument('x').value==std::string("included value"));
    ASSERT(validTestParameterParser.getUnpositionalArgument(std::string("long-argument-with-value-included")).value==std::string("included value"));
    ASSERT(!validTestParameterParser.getUnpositionalArgument('y').isError);
    ASSERT(validTestParameterParser.getUnpositionalArgument('y').value==std::string("separate value"));
    ASSERT(validTestParameterParser.getUnpositionalArgument(std::string("long-argument-with-value-separated")).value==std::string("separate value"));
    ASSERT(!validTestParameterParser.getPositionalArgument(1).isError);
    ASSERT(validTestParameterParser.getPositionalArgument(1).value=="positional argument 1");
    ASSERT(!validTestParameterParser.getPositionalArgument(2).isError);
    ASSERT(validTestParameterParser.getPositionalArgument(2).value=="positional argument 2");
    ASSERT(!validTestParameterParser.getUnpositionalArgument('z').isError);
    ASSERT(validTestParameterParser.getUnpositionalArgument('z').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(validTestParameterParser.getUnpositionalArgument('z').value == std::string(""));
    ASSERT(validTestParameterParser.wrongUsageMessages.size() == 0);

    delete[] validTestParameterList;
    std::cout << "Passed valid input test." << std::endl;

    // empty input (valid and invalid) #################################
    std::vector<CLI::Flag> emptyTestFlags;

    std::vector<CLI::UnpositionalArgument> emptyTestUnpositionalArguments;

    std::vector<CLI::PositionalArgument> emptyTestPositionalArguments;

    const char** emptyTestParameterList = new const char*[1];
    // The command is always a part of a command line, even if it is empty
    // otherwise. Therefore, I think it is reasonable to assume that it
    // will always be available.
    emptyTestParameterList[0] = "test";
    int emptyTestParameterCount = 1;

    // valid
    CLI::ArgumentsParser validEmptyTestParameterParser = CLI::ArgumentsParser(emptyTestParameterCount, emptyTestParameterList, emptyTestFlags, emptyTestUnpositionalArguments, emptyTestPositionalArguments);
    ASSERT(!validEmptyTestParameterParser.wrongUsage);
    ASSERT(validEmptyTestParameterParser.programName == std::string("test"));
    ASSERT(validEmptyTestParameterParser.wrongUsageMessages.size() == 0);

    //invalid
    emptyTestPositionalArguments.push_back(CLI::PositionalArgument("argument", "positional argument"));

    CLI::ArgumentsParser invalidEmptyTestParameterParser = CLI::ArgumentsParser(emptyTestParameterCount, emptyTestParameterList, emptyTestFlags, emptyTestUnpositionalArguments, emptyTestPositionalArguments);
    ASSERT(invalidEmptyTestParameterParser.wrongUsage);
    ASSERT(invalidEmptyTestParameterParser.programName == std::string("test"));
    ASSERT(invalidEmptyTestParameterParser.getPositionalArgument(0).isError);
    ASSERT(invalidEmptyTestParameterParser.getPositionalArgument(0).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(invalidEmptyTestParameterParser.wrongUsageMessages.size() == 1);
    ASSERT(invalidEmptyTestParameterParser.wrongUsageMessages.at(0) == "Too few positional arguments! Missing: argument");

    delete[] emptyTestParameterList;
    std::cout << "Passed empty input test." << std::endl;

    // unknown flag ####################################################

    std::vector<CLI::Flag> unknownFlagTestFlags;
    unknownFlagTestFlags.push_back(CLI::Flag('0', "zero", "test flag"));
    unknownFlagTestFlags.push_back(CLI::Flag('1', "one",  "test flag"));
    unknownFlagTestFlags.push_back(CLI::Flag('2', "two",  "test flag"));
    unknownFlagTestFlags.push_back(CLI::Flag('3', "three","test flag"));
    unknownFlagTestFlags.push_back(CLI::Flag('4', "four", "test flag"));
    unknownFlagTestFlags.push_back(CLI::Flag('5', "five", "test flag"));

    std::vector<CLI::UnpositionalArgument> unknownFlagTestUnpositionalArguments;

    std::vector<CLI::PositionalArgument> unknownFlagTestPositionalArguments;

    const char** unknownFlagTestParameterList = new const char*[7];
    unknownFlagTestParameterList[0] = "test";
    unknownFlagTestParameterList[1] = "-a";
    unknownFlagTestParameterList[2] = "-1";
    unknownFlagTestParameterList[3] = "-2";
    unknownFlagTestParameterList[4] = "-3";
    unknownFlagTestParameterList[5] = "-4";
    unknownFlagTestParameterList[6] = "-5";
    int unknownFlagStandaloneTestParameterCount = 2;

    CLI::ArgumentsParser unknownFlagStandaloneTestParser = CLI::ArgumentsParser(unknownFlagStandaloneTestParameterCount, unknownFlagTestParameterList, unknownFlagTestFlags, unknownFlagTestUnpositionalArguments, unknownFlagTestPositionalArguments);
    ASSERT(unknownFlagStandaloneTestParser.getFlag('1').isError);
    ASSERT(unknownFlagStandaloneTestParser.getFlag('1').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(!unknownFlagStandaloneTestParser.getFlag('1').value);
    ASSERT(unknownFlagStandaloneTestParser.getFlag(std::string("four")).isError);
    ASSERT(unknownFlagStandaloneTestParser.getFlag(std::string("four")).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(!unknownFlagStandaloneTestParser.getFlag(std::string("four")).value);
    ASSERT(unknownFlagStandaloneTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownFlagStandaloneTestParser.wrongUsageMessages.at(0) == "Unknown argument or flag(s): a");

    int unknownFlagMixedTestParameterCount = 7;

    CLI::ArgumentsParser unknownFlagMixedTestParser = CLI::ArgumentsParser(unknownFlagMixedTestParameterCount, unknownFlagTestParameterList, unknownFlagTestFlags, unknownFlagTestUnpositionalArguments, unknownFlagTestPositionalArguments);
    ASSERT(unknownFlagMixedTestParser.getFlag('1').isError);
    ASSERT(unknownFlagMixedTestParser.getFlag('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownFlagMixedTestParser.getFlag('1').value);
    ASSERT(unknownFlagMixedTestParser.getFlag(std::string("four")).isError);
    ASSERT(unknownFlagMixedTestParser.getFlag(std::string("four")).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownFlagMixedTestParser.getFlag(std::string("four")).value);
    ASSERT(unknownFlagMixedTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownFlagMixedTestParser.wrongUsageMessages.at(0) == "Unknown argument or flag(s): a");

    delete[] unknownFlagTestParameterList;

    std::vector<CLI::Flag> unknownLongFlagTestFlags;
    unknownLongFlagTestFlags.push_back(CLI::Flag('0', "zero", "test flag"));
    unknownLongFlagTestFlags.push_back(CLI::Flag('1', "one",  "test flag"));
    unknownLongFlagTestFlags.push_back(CLI::Flag('2', "two",  "test flag"));
    unknownLongFlagTestFlags.push_back(CLI::Flag('3', "three","test flag"));
    unknownLongFlagTestFlags.push_back(CLI::Flag('4', "four", "test flag"));
    unknownLongFlagTestFlags.push_back(CLI::Flag('5', "five", "test flag"));

    std::vector<CLI::UnpositionalArgument> unknownLongFlagTestUnpositionalArguments;

    std::vector<CLI::PositionalArgument> unknownLongFlagTestPositionalArguments;

    const char** unknownLongFlagTestParameterList = new const char*[7];
    unknownLongFlagTestParameterList[0] = "test";
    unknownLongFlagTestParameterList[1] = "--a";
    unknownLongFlagTestParameterList[2] = "-1";
    unknownLongFlagTestParameterList[3] = "-2";
    unknownLongFlagTestParameterList[4] = "-3";
    unknownLongFlagTestParameterList[5] = "-4";
    unknownLongFlagTestParameterList[6] = "-5";
    int unknownLongFlagStandaloneTestParameterCount = 2;

    CLI::ArgumentsParser unknownLongFlagStandaloneTestParser = CLI::ArgumentsParser(unknownLongFlagStandaloneTestParameterCount, unknownLongFlagTestParameterList, unknownLongFlagTestFlags, unknownLongFlagTestUnpositionalArguments, unknownLongFlagTestPositionalArguments);
    ASSERT(unknownLongFlagStandaloneTestParser.getFlag('1').isError);
    ASSERT(unknownLongFlagStandaloneTestParser.getFlag('1').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(!unknownLongFlagStandaloneTestParser.getFlag('1').value);
    ASSERT(unknownLongFlagStandaloneTestParser.getFlag(std::string("four")).isError);
    ASSERT(unknownLongFlagStandaloneTestParser.getFlag(std::string("four")).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(!unknownLongFlagStandaloneTestParser.getFlag(std::string("four")).value);
    ASSERT(unknownLongFlagStandaloneTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownLongFlagStandaloneTestParser.wrongUsageMessages.at(0) == "Unknown argument or flag: --a");

    int unknownLongFlagMixedTestParameterCount = 7;

    CLI::ArgumentsParser unknownLongFlagMixedTestParser = CLI::ArgumentsParser(unknownLongFlagMixedTestParameterCount, unknownLongFlagTestParameterList, unknownLongFlagTestFlags, unknownLongFlagTestUnpositionalArguments, unknownLongFlagTestPositionalArguments);
    ASSERT(unknownLongFlagMixedTestParser.getFlag('1').isError);
    ASSERT(unknownLongFlagMixedTestParser.getFlag('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownLongFlagMixedTestParser.getFlag('1').value);
    ASSERT(unknownLongFlagMixedTestParser.getFlag(std::string("four")).isError);
    ASSERT(unknownLongFlagMixedTestParser.getFlag(std::string("four")).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownLongFlagMixedTestParser.getFlag(std::string("four")).value);
    ASSERT(unknownLongFlagMixedTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownLongFlagMixedTestParser.wrongUsageMessages.at(0) == "Unknown argument or flag: --a");

    delete[] unknownLongFlagTestParameterList;
    std::cout << "Passed unknown flag test." << std::endl;

    // unknown unpositional argument ###################################

    std::vector<CLI::Flag> unknownArgumentTestFlags;

    std::vector<CLI::UnpositionalArgument> unknownArgumentTestUnpositionalArguments;
    unknownArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('0', "zero", "zero", "test argument"));
    unknownArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('1', "one",  "one",  "test argument"));
    unknownArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('2', "two",  "two",  "test argument"));
    unknownArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('3', "three","three","test argument"));
    unknownArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('4', "four", "four", "test argument"));
    unknownArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('5', "five", "five", "test argument"));

    std::vector<CLI::PositionalArgument> unknownArgumentTestPositionalArguments;

    const char** unknownArgumentTestParameterList = new const char*[7];
    unknownArgumentTestParameterList[0] = "test";
    unknownArgumentTestParameterList[1] = "-a123";
    unknownArgumentTestParameterList[2] = "-1a";
    unknownArgumentTestParameterList[3] = "-2b";
    unknownArgumentTestParameterList[4] = "-3c";
    unknownArgumentTestParameterList[5] = "-4d";
    unknownArgumentTestParameterList[6] = "-5e";
    int unknownArgumentStandaloneTestParameterCount = 2;

    CLI::ArgumentsParser unknownArgumentStandaloneTestParser = CLI::ArgumentsParser(unknownArgumentStandaloneTestParameterCount, unknownArgumentTestParameterList, unknownArgumentTestFlags, unknownArgumentTestUnpositionalArguments, unknownArgumentTestPositionalArguments);
    ASSERT(unknownArgumentStandaloneTestParser.getUnpositionalArgument('1').isError);
    ASSERT(unknownArgumentStandaloneTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(unknownArgumentStandaloneTestParser.getUnpositionalArgument('1').value == "");
    ASSERT(unknownArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(unknownArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(unknownArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).value == "");
    ASSERT(unknownArgumentStandaloneTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownArgumentStandaloneTestParser.wrongUsageMessages.at(0) == "Unknown argument or flag(s): a123");

    int unknownArgumentMixedTestParameterCount = 7;

    CLI::ArgumentsParser unknownArgumentMixedTestParser = CLI::ArgumentsParser(unknownArgumentMixedTestParameterCount, unknownArgumentTestParameterList, unknownArgumentTestFlags, unknownArgumentTestUnpositionalArguments, unknownArgumentTestPositionalArguments);
    ASSERT(unknownArgumentMixedTestParser.getUnpositionalArgument('1').isError);
    ASSERT(unknownArgumentMixedTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownArgumentMixedTestParser.getUnpositionalArgument('1').value == "a");
    ASSERT(unknownArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(unknownArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).value == "d");
    ASSERT(unknownArgumentMixedTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownArgumentMixedTestParser.wrongUsageMessages.at(0) == "Unknown argument or flag(s): a123");

    delete[] unknownArgumentTestParameterList;

    std::vector<CLI::Flag> unknownLongArgumentTestFlags;

    std::vector<CLI::UnpositionalArgument> unknownLongArgumentTestUnpositionalArguments;
    unknownLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('0', "zero", "zero", "test argument"));
    unknownLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('1', "one",  "one",  "test argument"));
    unknownLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('2', "two",  "two",  "test argument"));
    unknownLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('3', "three","three","test argument"));
    unknownLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('4', "four", "four", "test argument"));
    unknownLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('5', "five", "five", "test argument"));

    std::vector<CLI::PositionalArgument> unknownLongArgumentTestPositionalArguments;

    const char** unknownLongArgumentTestParameterList = new const char*[7];
    unknownLongArgumentTestParameterList[0] = "test";
    unknownLongArgumentTestParameterList[1] = "--a=123";
    unknownLongArgumentTestParameterList[2] = "-1a";
    unknownLongArgumentTestParameterList[3] = "-2b";
    unknownLongArgumentTestParameterList[4] = "-3c";
    unknownLongArgumentTestParameterList[5] = "-4d";
    unknownLongArgumentTestParameterList[6] = "-5e";
    int unknownLongArgumentStandaloneTestParameterCount = 2;

    CLI::ArgumentsParser unknownLongArgumentStandaloneTestParser = CLI::ArgumentsParser(unknownLongArgumentStandaloneTestParameterCount, unknownLongArgumentTestParameterList, unknownLongArgumentTestFlags, unknownLongArgumentTestUnpositionalArguments, unknownLongArgumentTestPositionalArguments);
    ASSERT(unknownLongArgumentStandaloneTestParser.getUnpositionalArgument('1').isError);
    ASSERT(unknownLongArgumentStandaloneTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(unknownLongArgumentStandaloneTestParser.getUnpositionalArgument('1').value == "");
    ASSERT(unknownLongArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(unknownLongArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(unknownLongArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).value == "");
    ASSERT(unknownLongArgumentStandaloneTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownLongArgumentStandaloneTestParser.wrongUsageMessages.at(0) == "Unknown argument (or it's a flag that doesn't take a value): --a=123");

    int unknownLongArgumentMixedTestParameterCount = 7;

    CLI::ArgumentsParser unknownLongArgumentMixedTestParser = CLI::ArgumentsParser(unknownLongArgumentMixedTestParameterCount, unknownLongArgumentTestParameterList, unknownLongArgumentTestFlags, unknownLongArgumentTestUnpositionalArguments, unknownLongArgumentTestPositionalArguments);
    ASSERT(unknownLongArgumentMixedTestParser.getUnpositionalArgument('1').isError);
    ASSERT(unknownLongArgumentMixedTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownLongArgumentMixedTestParser.getUnpositionalArgument('1').value == "a");
    ASSERT(unknownLongArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(unknownLongArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(unknownLongArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).value == "d");
    ASSERT(unknownLongArgumentMixedTestParser.wrongUsageMessages.size() == 1);
    ASSERT(unknownLongArgumentMixedTestParser.wrongUsageMessages.at(0) == "Unknown argument (or it's a flag that doesn't take a value): --a=123");

    delete[] unknownLongArgumentTestParameterList;
    std::cout << "Passed unknown unpositional argument test." << std::endl;

    // incomplete unpositional argument ###################################

    std::vector<CLI::Flag> incompleteArgumentTestFlags;

    std::vector<CLI::UnpositionalArgument> incompleteArgumentTestUnpositionalArguments;
    incompleteArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('0', "zero", "zero", "test argument"));
    incompleteArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('1', "one",  "one",  "test argument"));
    incompleteArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('2', "two",  "two",  "test argument"));
    incompleteArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('3', "three","three","test argument"));
    incompleteArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('4', "four", "four", "test argument"));
    incompleteArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('5', "five", "five", "test argument"));

    std::vector<CLI::PositionalArgument> incompleteArgumentTestPositionalArguments;

    const char** incompleteArgumentTestParameterList = new const char*[6];
    incompleteArgumentTestParameterList[0] = "test";
    incompleteArgumentTestParameterList[1] = "-1";
    incompleteArgumentTestParameterList[2] = "--two="; // value ""
    incompleteArgumentTestParameterList[3] = "-3c";
    incompleteArgumentTestParameterList[4] = "-4";
    incompleteArgumentTestParameterList[5] = "-5e";
    int incompleteArgumentStandaloneTestParameterCount = 2;

    CLI::ArgumentsParser incompleteArgumentStandaloneTestParser = CLI::ArgumentsParser(incompleteArgumentStandaloneTestParameterCount, incompleteArgumentTestParameterList, incompleteArgumentTestFlags, incompleteArgumentTestUnpositionalArguments, incompleteArgumentTestPositionalArguments);
    ASSERT(incompleteArgumentStandaloneTestParser.getUnpositionalArgument('1').isError);
    ASSERT(incompleteArgumentStandaloneTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteArgumentStandaloneTestParser.getUnpositionalArgument('1').value == "");
    ASSERT(incompleteArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(incompleteArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(incompleteArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).value == "");
    ASSERT(incompleteArgumentStandaloneTestParser.wrongUsageMessages.size() == 1);
    ASSERT(incompleteArgumentStandaloneTestParser.wrongUsageMessages.at(0) == "Argument expects value but has none: one");

    int incompleteArgumentMixedTestParameterCount = 6;

    CLI::ArgumentsParser incompleteArgumentMixedTestParser = CLI::ArgumentsParser(incompleteArgumentMixedTestParameterCount, incompleteArgumentTestParameterList, incompleteArgumentTestFlags, incompleteArgumentTestUnpositionalArguments, incompleteArgumentTestPositionalArguments);
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument('1').isError);
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument('1').value == "");
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).value == "");
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument('5').isError);
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument('5').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteArgumentMixedTestParser.getUnpositionalArgument('5').value == "e");
    ASSERT(incompleteArgumentMixedTestParser.wrongUsageMessages.size() == 2);
    ASSERT(incompleteArgumentMixedTestParser.wrongUsageMessages.at(0) == "Argument expects value but has none: one");
    ASSERT(incompleteArgumentMixedTestParser.wrongUsageMessages.at(1) == "Argument expects value but has none: four");

    delete[] incompleteArgumentTestParameterList;

    std::vector<CLI::Flag> incompleteLongArgumentTestFlags;

    std::vector<CLI::UnpositionalArgument> incompleteLongArgumentTestUnpositionalArguments;
    incompleteLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('0', "zero", "zero", "test argument"));
    incompleteLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('1', "one",  "one",  "test argument"));
    incompleteLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('2', "two",  "two",  "test argument"));
    incompleteLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('3', "three","three","test argument"));
    incompleteLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('4', "four", "four", "test argument"));
    incompleteLongArgumentTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('5', "five", "five", "test argument"));

    std::vector<CLI::PositionalArgument> incompleteLongArgumentTestPositionalArguments;

    const char** incompleteLongArgumentTestParameterList = new const char*[8];
    incompleteLongArgumentTestParameterList[0] = "test";
    incompleteLongArgumentTestParameterList[1] = "--one";
    incompleteLongArgumentTestParameterList[2] = "--two";
    incompleteLongArgumentTestParameterList[3] = "b";
    incompleteLongArgumentTestParameterList[4] = "-3c";
    incompleteLongArgumentTestParameterList[5] = "positional_arg";
    incompleteLongArgumentTestParameterList[6] = "--four";
    incompleteLongArgumentTestParameterList[7] = "--five=e";
    int incompleteLongArgumentStandaloneTestParameterCount = 2;

    CLI::ArgumentsParser incompleteLongArgumentStandaloneTestParser = CLI::ArgumentsParser(incompleteLongArgumentStandaloneTestParameterCount, incompleteLongArgumentTestParameterList, incompleteLongArgumentTestFlags, incompleteLongArgumentTestUnpositionalArguments, incompleteLongArgumentTestPositionalArguments);
    ASSERT(incompleteLongArgumentStandaloneTestParser.getUnpositionalArgument('1').isError);
    ASSERT(incompleteLongArgumentStandaloneTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteLongArgumentStandaloneTestParser.getUnpositionalArgument('1').value == "");
    ASSERT(incompleteLongArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(incompleteLongArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(incompleteLongArgumentStandaloneTestParser.getUnpositionalArgument(std::string("four")).value == "");
    ASSERT(incompleteLongArgumentStandaloneTestParser.wrongUsageMessages.size() == 1);
    ASSERT(incompleteLongArgumentStandaloneTestParser.wrongUsageMessages.at(0) == "Argument expects value but has none: one");

    int incompleteLongArgumentMixedTestParameterCount = 8;
    incompleteLongArgumentTestPositionalArguments.push_back(CLI::PositionalArgument("arg", "argument"));

    CLI::ArgumentsParser incompleteLongArgumentMixedTestParser = CLI::ArgumentsParser(incompleteLongArgumentMixedTestParameterCount, incompleteLongArgumentTestParameterList, incompleteLongArgumentTestFlags, incompleteLongArgumentTestUnpositionalArguments, incompleteLongArgumentTestPositionalArguments);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('1').isError);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('1').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('1').value == "");
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('2').isError);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('2').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('2').value == "b");
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).isError);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument(std::string("four")).value == "");
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('5').isError);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('5').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(incompleteLongArgumentMixedTestParser.getUnpositionalArgument('5').value == "e");
    ASSERT(incompleteLongArgumentMixedTestParser.wrongUsageMessages.size() == 2);
    ASSERT(incompleteLongArgumentMixedTestParser.wrongUsageMessages.at(0) == "Argument expects value but has none: one");
    ASSERT(incompleteLongArgumentMixedTestParser.wrongUsageMessages.at(1) == "Argument expects value but has none: four");

    delete[] incompleteLongArgumentTestParameterList;
    std::cout << "Passed incomplete unpositional argument test." << std::endl;

    // too few positional arguments

    std::vector<CLI::Flag> tooFewArgumentsTestFlags;
    tooFewArgumentsTestFlags.push_back(CLI::Flag('a', "aaaaa", "test flag"));

    std::vector<CLI::UnpositionalArgument> tooFewArgumentsTestUnpositionalArguments;
    tooFewArgumentsTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('b', "bbbbb", "b", "test argument"));
    tooFewArgumentsTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('c', "ccccc", "c", "test argument"));
    tooFewArgumentsTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('d', "ddddd", "d", "test argument"));

    std::vector<CLI::PositionalArgument> tooFewArgumentsTestPositionalArguments;
    tooFewArgumentsTestPositionalArguments.push_back(CLI::PositionalArgument("argument0", "test argument"));
    tooFewArgumentsTestPositionalArguments.push_back(CLI::PositionalArgument("argument1", "test argument"));
    tooFewArgumentsTestPositionalArguments.push_back(CLI::PositionalArgument("argument2", "test argument"));
    tooFewArgumentsTestPositionalArguments.push_back(CLI::PositionalArgument("argument3", "test argument"));
    tooFewArgumentsTestPositionalArguments.push_back(CLI::PositionalArgument("argument4", "test argument"));

    const char** tooFewArgumentsTestParameterList = new const char*[10];
    tooFewArgumentsTestParameterList[0] = "test";
    tooFewArgumentsTestParameterList[1] = "positional_arg_0";
    tooFewArgumentsTestParameterList[2] = "-a";//
    tooFewArgumentsTestParameterList[3] = "-b";
    tooFewArgumentsTestParameterList[4] = "1";
    tooFewArgumentsTestParameterList[5] = "-c5";
    tooFewArgumentsTestParameterList[6] = "positional_arg_1";
    tooFewArgumentsTestParameterList[7] = "--ddddd";
    tooFewArgumentsTestParameterList[8] = "ddd";
    tooFewArgumentsTestParameterList[9] = "positional_arg_2";

    int tooFewArgumentsStandaloneTestParameterCount = 2;
    CLI::ArgumentsParser tooFewArgumentsStandaloneTestParser = CLI::ArgumentsParser(tooFewArgumentsStandaloneTestParameterCount, tooFewArgumentsTestParameterList, tooFewArgumentsTestFlags, tooFewArgumentsTestUnpositionalArguments, tooFewArgumentsTestPositionalArguments);

    ASSERT(tooFewArgumentsStandaloneTestParser.getPositionalArgument(0).isError);
    ASSERT(tooFewArgumentsStandaloneTestParser.getPositionalArgument(0).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooFewArgumentsStandaloneTestParser.getPositionalArgument(0).value == "positional_arg_0");
    ASSERT(tooFewArgumentsStandaloneTestParser.getPositionalArgument(1).isError);
    ASSERT(tooFewArgumentsStandaloneTestParser.getPositionalArgument(1).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooFewArgumentsStandaloneTestParser.getPositionalArgument(1).value == "");
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('b').isError);
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('b').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('b').value == "");
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('c').isError);
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('c').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('c').value == "");
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('d').isError);
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('d').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooFewArgumentsStandaloneTestParser.getUnpositionalArgument('d').value == "");
    ASSERT(tooFewArgumentsStandaloneTestParser.wrongUsageMessages.size() == 4);
    ASSERT(tooFewArgumentsStandaloneTestParser.wrongUsageMessages.at(0) == "Too few positional arguments! Missing: argument1");
    ASSERT(tooFewArgumentsStandaloneTestParser.wrongUsageMessages.at(1) == "Too few positional arguments! Missing: argument2");
    ASSERT(tooFewArgumentsStandaloneTestParser.wrongUsageMessages.at(2) == "Too few positional arguments! Missing: argument3");
    ASSERT(tooFewArgumentsStandaloneTestParser.wrongUsageMessages.at(3) == "Too few positional arguments! Missing: argument4");

    int tooFewArgumentsMixedTestParameterCount = 10;
    CLI::ArgumentsParser tooFewArgumentsMixedTestParser = CLI::ArgumentsParser(tooFewArgumentsMixedTestParameterCount, tooFewArgumentsTestParameterList, tooFewArgumentsTestFlags, tooFewArgumentsTestUnpositionalArguments, tooFewArgumentsTestPositionalArguments);

    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(0).isError);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(0).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(0).value == "positional_arg_0");
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(1).isError);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(1).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(1).value == "positional_arg_1");
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(2).isError);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(2).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(2).value == "positional_arg_2");
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(3).isError);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(3).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(3).value == "");
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(4).isError);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(4).errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooFewArgumentsMixedTestParser.getPositionalArgument(4).value == "");
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('b').isError);
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('b').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('b').value == "1");
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('c').isError);
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('c').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('c').value == "5");
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('d').isError);
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('d').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooFewArgumentsMixedTestParser.getUnpositionalArgument('d').value == "ddd");
    ASSERT(tooFewArgumentsMixedTestParser.wrongUsageMessages.size() == 2);
    ASSERT(tooFewArgumentsMixedTestParser.wrongUsageMessages.at(0) == "Too few positional arguments! Missing: argument3");
    ASSERT(tooFewArgumentsMixedTestParser.wrongUsageMessages.at(1) == "Too few positional arguments! Missing: argument4");

    delete[] tooFewArgumentsTestParameterList;
    std::cout << "Passed too few positional arguments test." << std::endl;

    // too many positional arguments

    std::vector<CLI::Flag> tooManyArgumentsTestFlags;
    tooManyArgumentsTestFlags.push_back(CLI::Flag('a', "aaaaa", "test flag"));

    std::vector<CLI::UnpositionalArgument> tooManyArgumentsTestUnpositionalArguments;
    tooManyArgumentsTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('b', "bbbbb", "b", "test argument"));
    tooManyArgumentsTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('c', "ccccc", "c", "test argument"));
    tooManyArgumentsTestUnpositionalArguments.push_back(CLI::UnpositionalArgument('d', "ddddd", "d", "test argument"));

    std::vector<CLI::PositionalArgument> tooManyArgumentsTestPositionalArguments;
    tooManyArgumentsTestPositionalArguments.push_back(CLI::PositionalArgument("argument0", "test argument"));

    const char** tooManyArgumentsTestParameterList = new const char*[10];
    tooManyArgumentsTestParameterList[0] = "test";
    tooManyArgumentsTestParameterList[1] = "positional_arg_0";
    tooManyArgumentsTestParameterList[2] = "positional_arg_1";
    tooManyArgumentsTestParameterList[3] = "-a";//
    tooManyArgumentsTestParameterList[4] = "-b";
    tooManyArgumentsTestParameterList[5] = "1";
    tooManyArgumentsTestParameterList[6] = "-c5";
    tooManyArgumentsTestParameterList[7] = "--ddddd";
    tooManyArgumentsTestParameterList[8] = "ddd";
    tooManyArgumentsTestParameterList[9] = "positional_arg_2";

    int tooManyArgumentsStandaloneTestParameterCount = 3;
    CLI::ArgumentsParser tooManyArgumentsStandaloneTestParser = CLI::ArgumentsParser(tooManyArgumentsStandaloneTestParameterCount, tooManyArgumentsTestParameterList, tooManyArgumentsTestFlags, tooManyArgumentsTestUnpositionalArguments, tooManyArgumentsTestPositionalArguments);

    ASSERT(tooManyArgumentsStandaloneTestParser.getPositionalArgument(0).isError);
    ASSERT(tooManyArgumentsStandaloneTestParser.getPositionalArgument(0).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooManyArgumentsStandaloneTestParser.getPositionalArgument(0).value == "positional_arg_0");
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('b').isError);
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('b').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('b').value == "");
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('c').isError);
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('c').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('c').value == "");
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('d').isError);
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('d').errorCode == ErrorCodes::NOT_PRESENT);
    ASSERT(tooManyArgumentsStandaloneTestParser.getUnpositionalArgument('d').value == "");
    ASSERT(tooManyArgumentsStandaloneTestParser.wrongUsageMessages.size() == 1);
    ASSERT(tooManyArgumentsStandaloneTestParser.wrongUsageMessages.at(0) == "Too many positional arguments! Unexpected encounter of: positional_arg_1");

    int tooManyArgumentsMixedTestParameterCount = 10;
    CLI::ArgumentsParser tooManyArgumentsMixedTestParser = CLI::ArgumentsParser(tooManyArgumentsMixedTestParameterCount, tooManyArgumentsTestParameterList, tooManyArgumentsTestFlags, tooManyArgumentsTestUnpositionalArguments, tooManyArgumentsTestPositionalArguments);

    ASSERT(tooManyArgumentsMixedTestParser.getPositionalArgument(0).isError);
    ASSERT(tooManyArgumentsMixedTestParser.getPositionalArgument(0).errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooManyArgumentsMixedTestParser.getPositionalArgument(0).value == "positional_arg_0");
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('b').isError);
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('b').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('b').value == "1");
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('c').isError);
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('c').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('c').value == "5");
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('d').isError);
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('d').errorCode == ErrorCodes::WRONG_USAGE);
    ASSERT(tooManyArgumentsMixedTestParser.getUnpositionalArgument('d').value == "ddd");
    ASSERT(tooManyArgumentsMixedTestParser.wrongUsageMessages.size() == 2);
    ASSERT(tooManyArgumentsMixedTestParser.wrongUsageMessages.at(0) == "Too many positional arguments! Unexpected encounter of: positional_arg_1");
    ASSERT(tooManyArgumentsMixedTestParser.wrongUsageMessages.at(1) == "Too many positional arguments! Unexpected encounter of: positional_arg_2");

    delete[] tooManyArgumentsTestParameterList;
    std::cout << "Passed too many positional arguments test." << std::endl;

    return 0;
}
